unit UpdCtrls;

{

  Copyright (c) 1997 Demidov Andre (andre@lion.tu-chel.ac.ru)

  TScaleImage
    Scale, transparent and loading from file (RLE and BMP formats)
    based on
        transparent bitmaps by  Michael Vincze (vincze@ti.com)
    and TScaleImage by Jeff Kinzer E-Mail: 102413,3557

  TUpdBevel
    My first component - large Bevels

  TNewSpeedBtn
    Cool looking rounded and circle buttons
}

interface

uses WinTypes, WinProcs, Messages, SysUtils, Classes, Controls, Menus,
  Graphics, StdCtrls, ExtCtrls, Dialogs, Buttons, rlebmp, dsgnintf, forms;

type

  TScaleStyle = (ssNormal, ssScale, ssStretchXY, ssStretchX, ssStretchY);

  TScaleImage = class(TGraphicControl)
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Loaded; override;
  private
    NPaint:integer;
    ScaleBmp: TBitmap;
    ScaleWidth,
    ScaleHeight: Integer;
    FScaleStyle: TScaleStyle;
    FBorder,
    FCenter: Boolean;
    FPicture: TPicture;
    FTransporent: Boolean;
    FFileName: string;
    procedure LoadPicture;
    procedure SetScaleStyle(Value: TScaleStyle);
    procedure SetCenter(Value: Boolean);
    procedure SetPicture(Value: TPicture);
    procedure SetBorder(Value: Boolean);
    procedure SetTransporent(Value: Boolean);
    procedure SetFileName(Value: string);
    function StorePicture:Boolean;
  protected
    { Protected declarations }
    procedure Paint; override;
{    procedure WMPaint(var Message: TWMPaint); message WM_PAINT;}
  public
    procedure NeedRepaint;
  published
    { Published declarations }
    property FileName: string read FFileName write SetFileName;
    property Picture: TPicture read FPicture write SetPicture stored StorePicture;
    property ScaleStyle: TScaleStyle read FScaleStyle write SetScaleStyle default ssNormal;
    property Center: Boolean read FCenter write SetCenter default True;
    property Align;
    property Color;
    property Border: Boolean read FBorder write SetBorder default True;
    property Transporent: Boolean read FTransporent write SetTransporent
         default False;
    property DragCursor;
    property DragMode;
    property PopupMenu;
    property ParentColor;
    property ParentShowHint;
    property ShowHint;
    property Visible;
    property Enabled;
    property OnClick;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

  TBevelLevel = (blStandard, blSmall, blBig, blXtraBig);

  TUpdBevel = class(TBevel)
  private
    FLevel: TBevelLevel;
    procedure SetLevel(Value: TBevelLevel);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property Align;
    property ParentShowHint;
    property Shape;
    property ShowHint;
    property Style;
    property Visible;
    property Level: TBevelLevel read FLevel write SetLevel;
  end;

  TNewSpeedBtn = class(TSpeedButton)
  private
    FColor:TColor;
    FRound:Integer;
    procedure SetColor(Value:TColor);
    procedure SetRound(Value:Integer);
  protected
    procedure Paint; override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property AllowAllUp;
    property GroupIndex;
    property Down;
    property Caption;
    property Color:TColor read FColor write SetColor;
    property Round:Integer read FRound write SetRound;
    property Enabled;
    property Font;
    property Glyph;
    property Layout;
    property Margin;
    property NumGlyphs;
    property ParentFont;
    property ParentShowHint;
    property ShowHint;
    property Spacing;
    property Visible;
    property OnClick;
    property OnDblClick;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
  end;

procedure Register;

implementation

type
  TSIFilenameProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

{ TSIFilenameProperty }

procedure TSIFilenameProperty.Edit;
var
  SIFileOpen: TOpenDialog;
begin
  SIFileOpen := TOpenDialog.Create(Application);
  SIFileOpen.Filename := GetValue;
  SIFileOpen.Filter := 'Windows Bitmaps (*.BMP,*.RLE)|*.BMP;*.RLE';
  SIFileOpen.Options := SIFileOpen.Options + [ofPathMustExist,
    ofFileMustExist];
  try
    if SIFileOpen.Execute then SetValue(SIFileOpen.Filename);
  finally
    SIFileOpen.Free;
  end;
end;

function TSIFilenameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog];
end;


procedure Register;
begin
  RegisterComponents('Samples', [TUpdBevel,TScaleImage,TNewSpeedBtn]);
  RegisterPropertyEditor(TypeInfo(string), TScaleImage, 'Filename', TSIFilenameProperty);
end;

constructor TNewSpeedBtn.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FColor := clBtnFace;
  FRound := 6;
end;

procedure TNewSpeedBtn.SetColor(Value:TColor);
begin
  if FColor<>Value then
  begin
    FColor:=Value;
    Invalidate;
  end;
end;

procedure TNewSpeedBtn.SetRound(Value:Integer);
begin
  if FRound<>Value then
  begin
    FRound:=Value;
    if FRound>Height div 2 then
      Fround:=Height div 2;
    if Fround<=0 then
      Fround:=1;
    Invalidate;
  end;
end;

procedure DrawButtonText2(Canvas: TCanvas; const Caption: string;
  TextBounds: TRect; State: TButtonState);
var
  CString: array[0..255] of Char;
begin
  StrPCopy(CString, Caption);
  Canvas.Brush.Style := bsClear;
  if State = bsDisabled then
  begin
    with Canvas do
    begin
      OffsetRect(TextBounds, 1, 1);
      Font.Color := clWhite;
      DrawText(Handle, CString, Length(Caption), TextBounds, 0);
      OffsetRect(TextBounds, -1, -1);
      Font.Color := clDkGray;
      DrawText(Handle, CString, Length(Caption), TextBounds, 0);
    end;
  end
  else
    DrawText(Canvas.Handle, CString, -1, TextBounds,
      DT_CENTER or DT_VCENTER or DT_SINGLELINE);
end;

procedure TNewSpeedBtn.Paint;
Var HColor,SColor:TColor;
    GIcon:TBitMap;
    H,W,W1,X,X1,Y:Integer;
    TextBounds:TRect;
    CString: array[0..255] of Char;
function GetHighlightColor(C:TColor):Tcolor;
begin
  if C=clBtnFace then
    GetHighlightColor:=clBtnHighLight
  else if C=clBlack then
    GetHighlightColor:=clGray
  else if C=clMaroon then
    GetHighlightColor:=clRed
  else if C=clGreen then
    GetHighlightColor:=clLime
  else if C=clOlive then
    GetHighlightColor:=clYellow
  else if C=clNavy then
    GetHighlightColor:=clBlue
  else if C=clPurple then
    GetHighlightColor:=clFuchsia
  else if C=clTeal then
    GetHighlightColor:=clAqua
  else if (C=clSilver) or (C=clLime) or (C=clYellow)
     or (C=clAqua) or (C=clFuchsia) then
    GetHighlightColor:=clWhite
  else if C=clGray then
    GetHighlightColor:=clSilver
  else if C=clRed then
    GetHighlightColor:=clFuchsia
  else if C=clBlue then
    GetHighlightColor:=clAqua
  else
    GetHighlightColor:=clBtnHighlight;
end;
function GetShadowColor(C:TColor):Tcolor;
begin
  if C=clBtnFace then
    GetShadowColor:=clBtnShadow
  else if C=clGray then
    GetShadowColor:=clBlack
  else if C=clRed then
    GetShadowColor:=clMaroon
  else if C=clLime then
    GetShadowColor:=clGreen
  else if C=clYellow then
    GetShadowColor:=clOlive
  else if C=clBlue then
    GetShadowColor:=clNavy
  else if C=clFuchsia then
    GetShadowColor:=clPurple
  else if C=clAqua then
    GetShadowColor:=clTeal
  else if (C=clGray) or (C=clMaroon) or (C=clGreen)
     or (C=clNavy) or (C=clPurple) then
    GetShadowColor:=clBlack
  else if C=clSilver then
    GetShadowColor:=clGray
  else if C=clOlive then
    GetShadowColor:=clMaroon
  else if C=clWhite then
    GetShadowColor:=clSilver
  else
    GetShadowColor:=clBtnShadow;
end;
begin
  if FRound>Height div 2 then
    Fround:=Height div 2;
  if not Enabled and not (csDesigning in ComponentState) then
  begin
    FState := bsDisabled;
  end
  else if FState = bsDisabled then
    FState := bsUp;
  HColor:=GetHighlightColor(Fcolor);
  SColor:=GetShadowColor(FColor);
  Canvas.Font := Self.Font;
  if Length(Caption) > 0 then
  begin
    TextBounds := Rect(0, 0, Width-8, 0);
    DrawText(Canvas.Handle, StrPCopy(CString, Caption), -1, TextBounds,
      DT_CALCRECT);
  end
  else
    TextBounds := Rect(0, 0, 0, 0);
  With Canvas do
  begin
     Brush.Color:=FColor;
     Pen.Color:=FColor;
     FillRect(Rect(fRound,0,Width-fRound,Height));
     Pie(0,0,fRound*2,Height,fRound,0,fRound,Height);
     Pie(width-fround*2,0,width,Height,
          width-fRound,height,width-fRound,0);
     {if FState<>bsDisabled then
     begin }
       if Fstate<>bsDown then
         Pen.Color:=HColor
       else
         Pen.Color:=SColor;
       Arc(1,1,fRound*2-1,Height-1,fRound,1,1,Height-1);
       Arc(width-fround*2+1,1,width-1,Height-1,
          width-1,1,width-fRound,1);
       MoveTo(fRound,1);
       LineTo(width-fround,1);
       if Fstate=bsDown then
         Pen.Color:=HColor
       else
         Pen.Color:=SColor;
       Arc(width-fround*2+1,1,width-1,Height-1,
          width-fRound,height-1,width-1,1);
       Arc(1,1,fRound*2-1,Height-1,1,Height-1,fround,Height-1);
       MoveTo(fRound,height-2);
       LineTo(width-fround,height-2);
     {end; }
     Pen.Color:=clBlack;
     MoveTo(fRound,0);
     LineTo(width-fround,0);
     MoveTo(fRound,height-1);
     LineTo(width-fround,height-1);
     Arc(0,0,fRound*2,Height,fRound,0,fRound,Height);
     Arc(width-fround*2,0,width,Height,
          width-fRound,height,width-fRound,0);
     GIcon:=Glyph;
     if (GIcon<>nil) and (GIcon.Height>0) and (GIcon.Width>0) then
     begin
       Brush.Color:=FColor;
       W:=GIcon.Width div NumGlyphs;
       H:=GIcon.Height;
       W1:=Width-8;
       if W>W1 then
         W:=W1;
       if H>Height-4 then
         H:=Height-4;
       X:=0;
       if (FState=bsDisabled) and (numGlyphs>1) then
         X:=W;
       Y:= (Height-4-H) div 2;
       if Y<0 then Y:=0;
       X1:=(Width-8-w) div 2;
       if Length(Caption) > 0 then
          X1:=X1-(TextBounds.Right-TextBounds.Left+5) div 2;
       if X1<0 then X1:=0;
       if Fstate=bsDown then
       begin
         X1:=X1+1;
         Y:=Y+1;
       end;
       BrushCopy(Rect(X1+4,Y+2,X1+4+W,Y+2+H),
            GIcon,Rect(X,0,X+W,H),clOlive);
       Y:= (Height-4-TextBounds.Bottom-TextBounds.Top+1) div 2;
       if Y<0 then Y:=0;
       if Fstate=bsDown then
       begin
         Y:=Y+1;
       end;
      OffsetRect(TextBounds, X1+8+W, Y+2);
     end
     else
     begin
       X1:=(Width-8-TextBounds.Right-TextBounds.Left+1) div 2;
       if X1<0 then X1:=0;
       Y:= (Height-4-TextBounds.Bottom-TextBounds.Top+1) div 2;
       if Y<0 then Y:=0;
       if Fstate=bsDown then
       begin
         X1:=X1+1;
         Y:=Y+1;
       end;
       OffsetRect(TextBounds, X1+4, Y+2);
     end;
  end;
  if Length(Caption) > 0 then
  begin
    if TextBounds.Bottom>Height-2 then
       TextBounds.Bottom:=Height-2;
    if TextBounds.Right>Width-4 then
       TextBounds.Right:=Width-4;
    DrawButtonText2(Canvas,Caption,TextBounds,FState);
  end;
end;

constructor TScaleImage.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    TWinControl(AOwner).InsertControl(Self);
    ControlStyle := ControlStyle + [csOpaque];
    ScaleBmp := TBitmap.Create;
    FPicture := TPicture.Create;
    FCenter := True;
    FBorder := True;
    FTransporent:=False;
    FScaleStyle := ssNormal;
    Height := 100;
    Width := 100;
    ScaleWidth := -1;
    ScaleHeight := -1;
    FFileName:='';
    NPaint:=0;
end;

procedure TScaleImage.Loaded;
begin
  inherited Loaded;
  ScaleWidth := -1;
  ScaleHeight := -1;
  if FFileName<>'' then
    LoadPicture;
  nPaint:=0;
end;

function TScaleImage.StorePicture:Boolean;
begin
  Result:=False;
  if FFileName='' then
    Result:=True;
end;

destructor TScaleImage.Destroy;
begin
    ScaleBmp.Destroy;
    FPicture.Destroy;
    Inherited Destroy;
end;

procedure TScaleImage.SetScaleStyle(Value: TScaleStyle);
begin
    if FScaleStyle <> Value then
    begin
        FScaleStyle := Value;
        Invalidate;
    end;
end;

procedure TScaleImage.SetCenter(Value: Boolean);
begin
    if FCenter <> Value then
    begin
        FCenter := Value;
        Invalidate;
    end;
end;

procedure TScaleImage.SetFileName(Value: string);
begin
  if FFileName <> Value then
  begin
    FFileName := Value;
    if FFileName<>'' then
    begin
      LoadPicture;
      ScaleWidth:=-1;
      ScaleHeight:=-1;
      Invalidate;
    end;
  end;
end;

procedure TScaleImage.SetPicture(Value: TPicture);
begin
    if not((Value=nil) or (Value.Graphic=nil)) then
    if not(Value.Graphic is TBitmap) then
    begin
        MessageDlg('You may only use .BMP images.',mtWarning,[mbOK],0);
        Exit;
    end;
    if FPicture <> Value then
    begin
      FPicture.Assign(Value);
      ScaleWidth:=-1;
      ScaleHeight:=-1;
      Invalidate;
    end;
end;

procedure TScaleImage.SetBorder(Value: Boolean);
begin
    if FBorder <> Value then
    begin
      FBorder := Value;
      Invalidate;
    end;
end;

procedure TScaleImage.SetTransporent(Value: Boolean);
begin
    if FTransporent <> Value then
    begin
        FTransporent := Value;
        Invalidate;
    end;
end;

procedure TScaleImage.LoadPicture;
var newname:string;
    i,lst:integer;
begin
  if FFileName<>'' then
  begin
    if FileExists(FFileName) then
      FPicture.LoadFromFile(FFileName);
    if FFileName[2]=':' then
      newname:=FFilename
    else
    begin
      lst:=0;
      newname:=Application.ExeName;
      for i:=1 to length(newname) do
        if newname[i]='\' then
          lst:=i;
      if lst=0 then
        newname:=FFileName
      else
        newname:=Copy(newname,1,lst)+FFileName;
    end;
    if FileExists(newname) then
      FPicture.LoadFromFile(newname);
  end;
end;

procedure DrawTransparentBitmap (ahdc: HDC;
                                 Image: TBitmap;
                                 TransparentColor: TColor;
                                 xStart, yStart: Word);
var
  cColor          : TColorRef;
  bmAndBack,
  bmAndObject,
  bmAndMem,
  bmSave,
  bmBackOld,
  bmObjectOld,
  bmMemOld,
  bmSaveOld       : HBitmap;
  hdcMem,
  hdcBack,
  hdcObject,
  hdcTemp,
  hdcSave         : HDC;
  ptSize          : TPoint;
begin
{ set the transparent color to be the lower left pixel of the bitmap
}
hdcTemp := CreateCompatibleDC (ahdc);
SelectObject (hdcTemp, Image.Handle); { select the bitmap }

{ convert bitmap dimensions from device to logical points
}
ptSize.x := Image.Width;
ptSize.y := Image.Height;
DPtoLP (hdcTemp, ptSize, 1);  { convert from device logical points }

{ create some DCs to hold temporary data
}
hdcBack   := CreateCompatibleDC(ahdc);
hdcObject := CreateCompatibleDC(ahdc);
hdcMem    := CreateCompatibleDC(ahdc);
hdcSave   := CreateCompatibleDC(ahdc);

{ create a bitmap for each DC
}

{ monochrome DC
}
bmAndBack   := CreateBitmap (ptSize.x, ptSize.y, 1, 1, nil);
bmAndObject := CreateBitmap (ptSize.x, ptSize.y, 1, 1, nil);

bmAndMem    := CreateCompatibleBitmap (ahdc, ptSize.x, ptSize.y);
bmSave      := CreateCompatibleBitmap (ahdc, ptSize.x, ptSize.y);

{ each DC must select a bitmap object to store pixel data
}
bmBackOld   := SelectObject (hdcBack, bmAndBack);
bmObjectOld := SelectObject (hdcObject, bmAndObject);
bmMemOld    := SelectObject (hdcMem, bmAndMem);
bmSaveOld   := SelectObject (hdcSave, bmSave);

{ set proper mapping mode
}
SetMapMode (hdcTemp, GetMapMode (ahdc));

{ save the bitmap sent here, because it will be overwritten
}
BitBlt (hdcSave, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

{ set the background color of the source DC to the color.
  contained in the parts of the bitmap that should be transparent
}
cColor := SetBkColor (hdcTemp, TransparentColor);

{ create the object mask for the bitmap by performing a BitBlt()
  from the source bitmap to a monochrome bitmap
}
BitBlt (hdcObject, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCCOPY);

{ set the background color of the source DC back to the original color
}
SetBkColor (hdcTemp, cColor);

{ create the inverse of the object mask
}
BitBlt (hdcBack, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, NOTSRCCOPY);

{ copy the background of the main DC to the destination
}
BitBlt (hdcMem, 0, 0, ptSize.x, ptSize.y, ahdc, xStart, yStart, SRCCOPY);

{ mask out the places where the bitmap will be placed
}
BitBlt (hdcMem, 0, 0, ptSize.x, ptSize.y, hdcObject, 0, 0, SRCAND);

{ mask out the transparent colored pixels on the bitmap
}
BitBlt (hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcBack, 0, 0, SRCAND);

{ XOR the bitmap with the background on the destination DC
}
BitBlt (hdcMem, 0, 0, ptSize.x, ptSize.y, hdcTemp, 0, 0, SRCPAINT);

{ copy the destination to the screen
}
BitBlt (ahdc, xStart, yStart, ptSize.x, ptSize.y, hdcMem, 0, 0, SRCCOPY);

{ place the original bitmap back into the bitmap sent here
}
BitBlt (hdcTemp, 0, 0, ptSize.x, ptSize.y, hdcSave, 0, 0, SRCCOPY);

{ delete the memory bitmaps
}
DeleteObject (SelectObject (hdcBack, bmBackOld));
DeleteObject (SelectObject (hdcObject, bmObjectOld));
DeleteObject (SelectObject (hdcMem, bmMemOld));
DeleteObject (SelectObject (hdcSave, bmSaveOld));

{ delete the memory DCs
}
DeleteDC (hdcMem);
DeleteDC (hdcBack);
DeleteDC (hdcObject);
DeleteDC (hdcSave);
DeleteDC (hdcTemp);
end;

procedure TScaleImage.NeedRepaint;
begin
  Invalidate;
  NPaint:=1;
end;

procedure TScaleImage.Paint;
var
    CenterW, CenterH: Integer;
    PictWidth,
    PictHeight: Integer;
    NewWidth,
    NewHeight: Integer;
    RatioW, RatioH, SizeRatio: Real;
    TmpBmp:TBitmap;
    OldBitmap : HBitmap;
    MemDC : HDC;
    hOldPal : HPalette;
    MonoBmp:TBitmap;
    crBack, crText: TColorRef;
begin
{   oldBitmap:=Fpicture.Bitmap.ReleaseHandle;
   Fpicture.bitmap.Handle:=OldBitmap;
  with canvas do
    Stretchdraw(Rect(0,0,Picture.Width,Picture.Height),Fpicture.Graphic);
  Exit;}
    Canvas.CopyMode := cmSrcCopy;
    Canvas.Pen.Color := Self.Color;
    Canvas.Brush.Color := Self.Color;
    if not FTransporent then
      Canvas.FillRect(ClientRect);
    if not FPicture.Bitmap.Empty then
    begin
       PictHeight := FPicture.Bitmap.Height;
       PictWidth := FPicture.Bitmap.Width;
       if FScaleStyle = ssNormal then
       begin
         NewHeight:=PictHeight;
         NewWidth:=PictWidth;
       end
       else if FScaleStyle = ssScale then
       begin
         RatioW := Width / PictWidth;
         RatioH := Height / PictHeight;
         if RatioW > RatioH then
           SizeRatio := RatioH
         else
           SizeRatio := RatioW;
         NewWidth := Trunc(PictWidth * SizeRatio);
         if NewWidth > Width then
            NewWidth := Width;
         NewHeight := Trunc(PictHeight * SizeRatio);
         if NewHeight > Height then
           NewHeight := Height;
       end
       else if FScaleStyle = ssStretchXY then
       begin
         NewHeight:=Height;
         NewWidth:=Width;
       end
       else if FScaleStyle = ssStretchX then
       begin
         NewHeight:=PictHeight;
         NewWidth:=Width;
       end
       else if FScaleStyle = ssStretchY then
       begin
         NewHeight:=Height;
         NewWidth:=PictWidth;
       end
       else
       begin
         NewHeight:=Height;
         NewWidth:=Width;
       end;
       {if (NewHeight<>ScaleHeight) or (NewWidth<>ScaleWidth) then}
       begin
         ScaleBmp.Assign(FPicture.Bitmap);
         ScaleBmp.Height := NewHeight;
         ScaleBmp.Width := NewWidth;
         ScaleHeight := NewHeight;
         ScaleWidth := NewWidth;
         if (NewHeight<>PictHeight) or (NewWidth<>PictWidth)  or (NPaint=0) then
         begin
           {hOldPal := SelectPalette(Canvas.handle,FPicture.Bitmap.Palette,False);
           RealizePalette(ScaleBmp.Canvas.handle);}
           MemDC := CreateCompatibleDC(Canvas.handle);
           OldBitmap := SelectObject(MemDC,FPicture.Bitmap.Handle);
           SetStretchBltMode(ScaleBmp.Canvas.Handle, STRETCH_DELETESCANS);
           StretchBlt(ScaleBmp.Canvas.Handle,0,0,NewWidth,NewHeight,
              MemDC {FPicture.Bitmap.Canvas.Handle} ,0,0,PictWidth,PictHeight,SRCCOPY);
           SelectObject(MemDC,OldBitmap);
           DeleteDC(MemDC);
           {SelectPalette(Canvas.handle,hOldPal,False);}
         end;
       end;
       if Center then
       begin
         CenterW := (Width - ScaleWidth) div 2;
         if CenterW<0 then CenterW:=0;
         CenterH := (Height - ScaleHeight) div 2;
         if CenterH<0 then CenterH:=0;
       end
       else
       begin
         CenterW := 0;
         CenterH := 0;
       end;

    if FTransporent then
    begin
      DrawTransparentBitmap(Canvas.handle,ScaleBmp,
         Fpicture.Bitmap.Canvas.pixels[0,0] or $02000000,
         CenterW,CenterH);
    end
    else
    begin
{
      hOldPal := SelectPalette(Canvas.handle,ScaleBmp.Palette,False);
      RealizePalette(Canvas.handle);

      updateColors(DC);
      MemDC := CreateCompatibleDC(Canvas.handle);
      OldBitmap := SelectObject(MemDC,ScaleBmp.Handle);
         BitBlt(Canvas.handle,
             CenterW,
             CenterH,
             ScaleBmp.Width,
             ScaleBmp.Height,
             MemDC,
             0,
             0,
             srcCopy);
      SelectObject(MemDC,OldBitmap);
      DeleteDC(MemDC);
      SelectPalette(Canvas.handle,hOldPal,False);
}
{        oldBitmap:=Scalebmp.ReleaseHandle;
        Scalebmp.Handle:=OldBitmap;}
        Canvas.Draw(CenterW,CenterH,ScaleBmp);
      end;
    end;
    Canvas.Brush.Color := clBlack;
    if FBorder then
       Canvas.FrameRect(ClientRect);
end;

{procedure TScaleImage.WMPaint(var Message: TWMPaint);
begin
    if Message.DC <> 0 then
    begin
        Canvas.Handle := Message.DC;
        try
            Paint;
        finally
            Canvas.Handle := 0;
        end;
    end;
end;
}

constructor TUpdBevel.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLevel := blStandard;
end;

procedure TUpdBevel.SetLevel(Value: TBevelLevel);
begin
  if Value <> FLevel then
  begin
    FLevel := Value;
    Invalidate;
  end;
end;

procedure TUpdBevel.Paint;
var
  Color1, Color2: TColor;
  Temp: TColor;
  w:integer;

  procedure UpdBevelHLine(C:TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      while Y1<>Y2 do
      begin
        MoveTo(X1,Y1);
        LineTo(X2,Y1);
        X1:=X1+1;
        X2:=X2-1;
        if Y1<Y2 then
          Y1:=Y1+1
        else
          Y1:=Y1-1;
      end;
      MoveTo(X1,Y1);
      LineTo(X2,Y1);
    end;
  end;
  procedure UpdBevelVLine(C: TColor; X1, Y1, X2, Y2: Integer);
  begin
    with Canvas do
    begin
      Pen.Color := C;
      while X1<>X2 do
      begin
        MoveTo(X1,Y1);
        LineTo(X1,Y2);
        Y1:=Y1+1;
        Y2:=Y2-1;
        if X1<X2 then
          X1:=X1+1
        else
          X1:=X1-1;
      end;
      MoveTo(X1,Y1);
      LineTo(X1,Y2);
    end;
  end;

  procedure UpdBevelRect(const R: TRect; w:integer);
  begin
    UpdBevelVLine(Color1,R.Left, R.Top,R.Left+w-1, R.Bottom);
    UpdBevelHLine(Color1,R.Left, R.Top,R.Right, R.Top+w-1);
    UpdBevelVLine(Color2,R.Right, R.Top,R.Right-w+1, R.Bottom);
    UpdBevelHLine(Color2,R.Left, R.Bottom,R.Right+1, R.Bottom-w+1);
  end;

begin
  if FLevel=blStandard then
    inherited Paint
  else
  with Canvas do
  begin
    Pen.Width := 1;
    if FLevel=blSmall then
      w:=2
    else if FLevel=blBig then
      w:=3
    else
      w:=4;
    if Style = bsLowered then
    begin
      Color1 := clBtnShadow;
      Color2 := clBtnHighlight;
    end
    else
    begin
      Color1 := clBtnHighlight;
      Color2 := clBtnShadow;
    end;
    case Shape of
      bsBox: UpdBevelRect(Rect(0, 0, Width - 1, Height - 1),w);
      bsFrame:
        begin
          UpdBevelRect(Rect(0, 0, Width - 1, Height - 1),w);
          Temp := Color1;
          Color1 := Color2;
          Color2 := Temp;
          UpdBevelRect(Rect(w, w, Width - 1 -w, Height - 1 - w),w);
        end;
      bsTopLine:
        begin
          UpdBevelHLine(Color1, 0, 0, Width-1, w-1);
          UpdBevelHLine(Color2, w, w, Width-w-1, w+w-1);
        end;
      bsBottomLine:
        begin
          UpdBevelHLine(Color1, w, Height - w-1, Width-w-1, Height - w-w);
          UpdBevelHLine(Color2, 0, Height - 1, Width-1, Height - w);
        end;
      bsLeftLine:
        begin
          UpdBevelVLine(Color1, 0, 0, w-1, Height-1);
          UpdBevelVLine(Color2, w, w, w+w-1, Height-1);
        end;
      bsRightLine:
        begin
          UpdBevelVLine(Color1, Width - w-1, w, Width - w-w, Height-w-1);
          UpdBevelVLine(Color2, Width - 1, 0, Width - w, Height-1);
        end;
    end;
  end;
end;

initialization
  TPicture.RegisterFileFormat('RLE','Windows bitmap files',TRLEBitmap);

end.
